/************************************************************************************
 * spu2spu - SPU ADPCM reinterleaver
 * Copytrighted by CaitSith2
 *
 * I wrote this for the purpose of reinterleaving ADPCM streams
 * for making streaming psf2s.  For the stream to work for
 * a psf2 using the released streaming IOP module, the stream has
 * to be 16384 bytes for the left channel and 16384 bytes for the right channel.
 *
 * Most adpcm streams use other sizes, like 16 for left/right, or 2048 for left/right
 *
 * Support for the general case (interleaves not evenly divisible) added by hcs (less efficient but it works)
 *
 ************************************************************************************/

#include <stdio.h>
#include <stdlib.h>

int main(int argc, char* argv[])
{
	FILE *f, *g;
	int infilepointer, outfilepointer;
	int block, blockout;

	unsigned char buf[16];

	printf("SPU ADPCM reinterleaver, v1.0hcsomglol\n");
	printf("Copyrighted by CaitSith2\nGeneral purpose recodage by hcs\n\n");


	if (argc < 3)
	{
		printf("Usage: spu2spu infile outfile [inblocksize] [outblocksize]\n\n");
		printf("default inblocksize = 16\n");
		printf("default outblocksize = 16384\n");
		return -1;
	}

	if (argc >= 4)
	{
		block=atoi(argv[3]);
	}
	else
	{
		block = 0;
	}
	if (argc == 5)
	{
		blockout=atoi(argv[4]);
	}
	else
	{
		blockout = 0;
	}

	if(block % 16 != 0)
	{
		printf("inblocksize must be a multiple of 16\n");
		return -1;
	}
	if (blockout % 16 != 0)
	{
		printf("outblocksize must be a multiple of 16\n");
		return -1;
	}
	if(block == 0)
	{
		block = 16;
	}
	if(blockout == 0)
	{
		blockout = 16384;
	}


	f=fopen(argv[1],"rb");
	g=fopen(argv[2], "wb");
	if (f == NULL)
	{
		printf("Error opening input file %s\n",argv[1]);
		return -1;
	}
	if (g == NULL)
	{
		printf("Error opening output file %s\n",argv[2]);
		return -1;
	}
	
	infilepointer=0;
	outfilepointer=0;
	
	/*if (block < block16)
	{
		while ((i=fgetc(f))!=EOF)
		{
			fseek(f,filepointer,SEEK_SET);
			for(i=0;i<(block16/block);i++)
			{
				fread(leftblock,1,block,f);
				fread(rightblock,1,block,f);
				for (j=0;j<block;j++)
				{
					if (j % 16 == 1)
					{
						leftblock[j] = 0;
						rightblock[j] = 0;
					}
					leftblock16[(i*block)+j]=leftblock[j];
					rightblock16[(i*block)+j]=rightblock[j];
				}
			}
			filepointer+=block16;
			filepointer+=block16;
			fwrite(leftblock16,1,block16,g);
			fwrite(rightblock16,1,block16,g);
		}
	}
	else
	{
		while ((i=fgetc(f))!=EOF)
		{
			fseek(f,filepointer,SEEK_SET);
			fread(leftblock,1,block,f);
			fread(rightblock,1,block,f);
			filepointer+=block;
			filepointer+=block;
			for(i=0;i<(block/block16);i++)
			{
				for (j=0;j<block16;j++)
				{
					leftblock16[j]=leftblock[(i*block16)+j];
					rightblock16[j]=rightblock[(i*block16)+j];
				}
				fwrite(leftblock16,1,block16,g);
				fwrite(rightblock16,1,block16,g);
			}
		}
					
	}*/

	while (fgetc(f)!=EOF) { // K&R rocks your world
		fseek(f,infilepointer,SEEK_SET);
		fseek(g,outfilepointer,SEEK_SET);
		fread(buf,1,16,f);
		fwrite(buf,1,16,g);

		fseek(f,infilepointer+block,SEEK_SET);
		fseek(g,outfilepointer+blockout,SEEK_SET);
		fread(buf,1,16,f);
		fwrite(buf,1,16,g);
		
		infilepointer+=16;
		outfilepointer+=16;
		if (!(infilepointer%block)) infilepointer+=block;
		if (!(outfilepointer%blockout)) outfilepointer+=blockout;
	}

	fclose(f);
	fclose(g);

	//free(leftblock);
	//free(rightblock);
	//free(leftblock16);
	//free(rightblock16);
	
	return 0;
}
